{{
  dwd_PWM120219a.spin
  Modified Kye's H-Bridge Demo to drive forward and then backward.
  Modifications by Duane Degn, 19 February, 2012.

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// PWM2C H-Bridge Demo
//
// Author: Kwabena W. Agyeman
// Updated: 8/20/2010
// Designed For: P8X32A
// Version: 1.0
//
// Copyright (c) 2010 Kwabena W. Agyeman
// See end of file for terms of use.
//
// Update History:
//
// v1.0 - Original release - 8/20/2010.
//
// Run the program with the specified driver hardware.
//
// Nyamekye,
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}

CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  _leftFW = 22 
  _leftBW = 23
  _rightFW = 20
  _rightBW = 21

  _LedPin = 16

  _ForwardRamp = 1
  _BackwardRamp = _ForwardRamp

  _DefaultForwardTime = 50 ' one second
  _DefaultBackwardTime = 55 ' 1.1 second

  _ForwardSpeed = 500 ' half speed for now
  _BackwardSpeed = _ForwardSpeed

  _timeStepInMilliseconds = 20  ' updates at 50Hz, similar to a hobby servo
  _updateFrequencyInHertz = 8_000

VAR

  long nextUpdateTime, updateInterval
  
OBJ

  hbd: "PWM2C_HBDEngine.spin"

PUB Setup

  ifnot(hbd.HBDEngineStart(_leftFW, _leftBW, _rightFW, _rightBW, _updateFrequencyInHertz))
    reboot

  updateInterval := ((clkfreq / 1_000) * _timeStepInMilliseconds)
  
  dira[_LedPin] := 1            ' LED pin an output
  
  Main

PUB Main


  DriveForward(_DefaultForwardTime)

  waitcnt(clkfreq / 2 + cnt)  ' wait half a second
  
  DriveBackward(_DefaultBackwardTime)

  repeat ' keep program running and flash LED to show program is done
    !outa[_LedPin]
    waitcnt(clkfreq / 2 + cnt)

PUB DriveForward(driveTime) | localPwmIndex

  nextUpdateTime := cnt + updateInterval
   
  repeat localPwmIndex from 0 to _ForwardSpeed step _ForwardRamp
    hbd.leftDuty(localPwmIndex)
    hbd.rightDuty(localPwmIndex)     
    waitcnt(nextUpdateTime)
    nextUpdateTime += updateInterval
    
  repeat driveTime
    waitcnt(nextUpdateTime)
    nextUpdateTime += updateInterval
    
  repeat localPwmIndex from _ForwardSpeed to 0 step _ForwardRamp
    hbd.leftDuty(localPwmIndex)
    hbd.rightDuty(localPwmIndex)     
    waitcnt(nextUpdateTime)
    nextUpdateTime += updateInterval
    
PUB DriveBackward(driveTime) | localPwmIndex
    
  nextUpdateTime := cnt + updateInterval
  
  repeat localPwmIndex from 0 to _BackwardSpeed step _BackwardRamp
    hbd.leftDuty(-localPwmIndex)
    hbd.rightDuty(-localPwmIndex)     
    waitcnt(nextUpdateTime)
    nextUpdateTime += updateInterval
    
  repeat driveTime
    waitcnt(nextUpdateTime)
    nextUpdateTime += updateInterval
    
  repeat localPwmIndex from _BackwardSpeed to 0 step _BackwardRamp
    hbd.leftDuty(-localPwmIndex)
    hbd.rightDuty(-localPwmIndex)     
    waitcnt(nextUpdateTime)
    nextUpdateTime += updateInterval
 

{{

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                  TERMS OF USE: MIT License
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
// Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
// COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}